/*
Copyright 2010-2011 Zhengmao HU (James)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package net.sf.jabb.util.stat;

/**
 * It stores the maximum value; 
 * the compare-and-change procedure is synchronized so that multi-thread safe is ensured.<br>
 * ֵȽȻ󽻻ĹͬģǶ̰߳ȫġ
 * 
 * @author Zhengmao HU (James)
 *
 */
public class AtomicMaxLong extends AtomicMinMaxLong {
	
	/**
	 * Constructs an instance to store the maximum value.<br>
	 * һʵֵ
	 */
	public AtomicMaxLong(){
		super(Long.MIN_VALUE);
	}
	
	/**
	 * Compare a value with current maximum value and make the greater one the new maximum value;
	 * New maximum value after comparison is returned.<br>
	 * һֵͬǰֵȽϣдǸΪµֵرȽ֮µֵ
	 * 
	 * @param newValue	Ƚϵֵ
	 * @return		New maximum value after comparison<br>Ƚ֮µֵ
	 */
	public long maxAndGet(long newValue){
		synchronized(updateLock){
			if (newValue > value){
				value = newValue;
			}
			return value;
		}
	}

	/**
	 * Compare a value with current maximum value and make the greater one the new maximum value;
	 * Previous maximum value before comparison is returned.<br>
	 * һֵͬǰֵȽϣдǸΪµֵرȽ֮ǰϵֵ
	 * 
	 * @param newValue	Ƚϵֵ
	 * @return		Previous maximum value before comparison<br>Ƚ֮ǰϵֵ
	 */
	public long getAndMax(long newValue){
		long oldValue;
		synchronized(updateLock){
			oldValue = value;
			if (newValue > value){
				value = newValue;
			}
			return oldValue;
		}
	}
	
	/**
	 * Compare a value with current maximum value and make the greater one the new maximum value.<br>
	 * һֵͬǰֵȽϣдǸΪµֵ
	 * 
	 * @param newValue	Ƚϵֵ
	 */
	public void max(long newValue){
		maxAndGet(newValue);
	}

	/**
	 * Reset to initial status.<br>
	 * ظʼ״̬
	 */
	@Override
	public void reset() {
		value = Long.MIN_VALUE;
	}


}
